
#include <iostream>
#include <stdlib.h>
#include <math.h>

#include <SDL.h>
#include <SDL_thread.h>

#include "pipes/streamPipe.h"
#include "elements/streamElement.h"
#include "elements/swarmElement.h"
#include "elements/tone.h"
#include "soundDev/soundDev.h"

#ifdef main
  #undef main
#endif

int
main( int argc, char* argv[] )
{
    unsigned int initFlags = 0U;

    initFlags |= SDL_INIT_AUDIO;

    if ( ::SDL_Init( initFlags ) < 0 ) {
	std::cerr << "Couldn't init SDL:  " << ::SDL_GetError() << std::endl;
	return __LINE__;
    }

    StreamPipe pB( 8192U * 7U * sizeof( double ) );
    StreamPipe pA( 32U * 128U * 3U * sizeof( unsigned char ) );

    SoundDev dev(
	&pB,		// input
	8192U,		// frequency
	1U,		// channels
	2U * 8192U,	// samples in the buffer
	1.0		// gain
    );

    Tone tone(
	&pA, &pB,	// input, output
	dev.getFreq(),	// frequency
	128.0,		// linesPerSecond
	1.0,		// gain
	Tone::SINE,	// wave form
	0.44,		// gamma
	128.0,		// minHz
	2048.0		// maxHz
    );

    SwarmElement swarm(
	&pA,		// input
	128.0,		// lines per second
	0,		// total lines
	10.0,		// gain
	false,		// doAbs
	32,		// dimensions
	7,		// gnats
	1.0		// accelFactor
    );

    swarm.startThread();
    tone.startThread();
    dev.startThread();

	// XXX could do other stuff here....

    dev.finish();

    dev.waitForThread();
    tone.waitForThread();
    swarm.waitForThread();

    ::SDL_Quit();

    return 0;
}
